@@ -73,6 +73,7 @@
 #include "MagickCore/option-private.h"
 #include "MagickCore/pixel-accessor.h"
 #include "MagickCore/profile.h"
+#include "MagickCore/profile-private.h"
 #include "MagickCore/property.h"
 #include "MagickCore/quantum-private.h"
 #include "MagickCore/resource_.h"
@@ -611,8 +612,8 @@ static boolean ReadICCProfile(j_decompress_ptr jpeg_info)
   client_info=(JPEGClientInfo *) jpeg_info->client_data;
   image=client_info->image;
   exception=client_info->exception;
-  status=SetImageProperty(image,"icc",(const char *) GetStringInfoDatum(
-    client_info->profiles[ICC_INDEX]),exception);
+  status=SetImageProfile(image,"icc",client_info->profiles[ICC_INDEX],
+    exception);
   return(status == MagickFalse ? FALSE : TRUE);
 }
 
@@ -689,8 +690,8 @@ static boolean ReadIPTCProfile(j_decompress_ptr jpeg_info)
   client_info=(JPEGClientInfo *) jpeg_info->client_data;
   image=client_info->image;
   exception=client_info->exception;
-  status=SetImageProperty(image,"iptc",(const char *) GetStringInfoDatum(
-    client_info->profiles[IPTC_INDEX]),exception);
+  status=SetImageProfile(image,"8bim",client_info->profiles[IPTC_INDEX],
+    exception);
   return(status == MagickFalse ? FALSE : TRUE);
 }
 
@@ -757,11 +758,10 @@ static boolean ReadAPPProfiles(j_decompress_ptr jpeg_info)
       }
       if (i != length)
         {
-          profile=AcquireStringInfo(length);
+          profile=AcquireProfileStringInfo("xmp",length,exception);
           (void) memcpy(GetStringInfoDatum(profile),p+1,length-i-1);
           SetStringInfoLength(profile,length-i-1);
-          status=SetImageProfile(image,"xmp",profile,exception);
-          profile=DestroyStringInfo(profile);
+          status=SetImageProfilePrivate(image,profile,exception);
           client_info->profiles[marker]=DestroyStringInfo(
             client_info->profiles[marker]);
         }
@@ -776,10 +776,9 @@ static boolean ReadAPPProfiles(j_decompress_ptr jpeg_info)
             /*
               Extract EXIF profile.
             */
-            profile=AcquireStringInfo(length);
+            profile=AcquireProfileStringInfo("exif",length,exception);
             (void) memcpy(GetStringInfoDatum(profile),p,length);
-            status=SetImageProfile(image,"exif",profile,exception);
-            profile=DestroyStringInfo(profile);
+            status=SetImageProfilePrivate(image,profile,exception);
             client_info->profiles[marker]=DestroyStringInfo(
               client_info->profiles[marker]);
           }
@@ -1167,12 +1166,19 @@ static Image *ReadOneJPEGImage(const ImageInfo *image_info,
   JPEGSourceManager(jpeg_info,image);
   jpeg_set_marker_processor(jpeg_info,JPEG_COM,ReadComment);
   option=GetImageOption(image_info,"profile:skip");
-  if (IsOptionMember("ICC",option) == MagickFalse)
-    jpeg_set_marker_processor(jpeg_info,ICC_MARKER,ReadICCProfile);
-  if (IsOptionMember("IPTC",option) == MagickFalse)
-    jpeg_set_marker_processor(jpeg_info,IPTC_MARKER,ReadIPTCProfile);
   for (i=1; i < MaxJPEGProfiles; i++)
-    if ((i != IPTC_INDEX) && (i != 14))
+  {
+    if (i == ICC_INDEX)
+      {
+        if (IsOptionMember("ICC",option) == MagickFalse)
+          jpeg_set_marker_processor(jpeg_info,ICC_MARKER,ReadICCProfile);
+      }
+    else if (i == IPTC_INDEX)
+      {
+        if (IsOptionMember("IPTC",option) == MagickFalse)
+          jpeg_set_marker_processor(jpeg_info,IPTC_MARKER,ReadIPTCProfile);
+      }
+    else if (i != 14)
       {
         /*
           Ignore APP14 as this will change the colors of the image.
@@ -1181,6 +1187,7 @@ static Image *ReadOneJPEGImage(const ImageInfo *image_info,
           jpeg_set_marker_processor(jpeg_info,(int) (JPEG_APP0+i),
             ReadAPPProfiles);
       }
+  }
   (void) jpeg_read_header(jpeg_info,TRUE);
   if (IsYCbCrCompatibleColorspace(image_info->colorspace) != MagickFalse)
     jpeg_info->out_color_space=JCS_YCbCr;
